@@ -2,8 +2,9 @@ import glob
 import json
 import tempfile
 from pathlib import Path
-from typing import Callable, Optional, Dict
+from typing import Callable, Optional, Dict, Tuple
 
+from keras import Model  # type: ignore
 from tensorflow.keras.models import load_model  # type: ignore
 import tarfile
 import ntpath
@@ -11,7 +12,6 @@ import ntpath
 import onnx  # type: ignore
 from onnx2kerastl import onnx_to_keras  # type: ignore
 
-
 from keras_data_format_converter import convert_channels_first_to_last  # type: ignore
 
 from leap_model_parser.contract.importmodelresponse import NodeResponse, ImportModelTypeEnum
@@ -29,24 +29,30 @@ class ModelParser:
             ImportModelTypeEnum.PB_TF2.value: self.convert_pb_model,
         }
 
-    def generate_model_graph(self, model_path: Path, model_type: ImportModelTypeEnum,
-                             should_transform_inputs=False) -> Dict[str, NodeResponse]:
+    def get_keras_model_and_model_graph(self, model_path: Path, model_type: ImportModelTypeEnum,
+                                        should_transform_inputs=False) -> Tuple[
+        Dict[str, NodeResponse], Optional[Model]]:
         self._should_transform_inputs = should_transform_inputs
         model_to_keras_converter: Optional[Callable[[str], Dict]] = self._model_types_converter.get(model_type.value)
         if model_to_keras_converter is None:
             raise Exception(f"Unable to import external version, {str(model_path)} file format isn't supported")
 
-        model_schema = model_to_keras_converter(str(model_path))
+        model_schema, keras_model = model_to_keras_converter(str(model_path))
         model_generator = KerasJsonModelImport()
-        return model_generator.generate_graph(model_schema)
+        return model_generator.generate_graph(model_schema), keras_model
+
+    def generate_model_graph(self, model_path: Path, model_type: ImportModelTypeEnum,
+                             should_transform_inputs=False) -> Dict[str, NodeResponse]:
+        model_graph, _ = self.get_keras_model_and_model_graph(model_path, model_type, should_transform_inputs)
+        return model_graph
 
     @classmethod
-    def convert_json_model(cls, file_path: str) -> Dict:
+    def convert_json_model(cls, file_path: str) -> Tuple[Dict[str, NodeResponse], None]:
         with open(file_path, 'r') as f:
             model_schema = json.load(f)
-        return model_schema
+        return model_schema, None
 
-    def convert_pb_model(self, file_path: str) -> Dict:
+    def convert_pb_model(self, file_path: str) -> Tuple[Dict[str, NodeResponse], Model]:
         tf = tarfile.open(file_path)
 
         with tempfile.TemporaryDirectory() as temp_dir:
@@ -61,7 +67,7 @@ class ModelParser:
 
         return self.convert_to_keras_model(k_model)
 
-    def convert_onnx_model(self, file_path: str) -> Dict:
+    def convert_onnx_model(self, file_path: str) -> Tuple[Dict[str, NodeResponse], Model]:
         onnx_model = onnx.load_model(file_path)
         input_names = [_input.name for _input in onnx_model.graph.input]
         k_model = onnx_to_keras(onnx_model, input_names=input_names,
@@ -69,14 +75,12 @@ class ModelParser:
 
         return self.convert_to_keras_model(k_model)
 
-    @staticmethod
-    def convert_h5_model(file_path: str) -> Dict:
+    def convert_h5_model(self, file_path: str) -> Tuple[Dict[str, NodeResponse], Model]:
         imported_model = load_model(file_path)
-        model_schema = json.loads(imported_model.to_json())
 
-        return model_schema
+        return self.convert_to_keras_model(imported_model)
 
-    def convert_to_keras_model(self, k_model) -> Dict:
+    def convert_to_keras_model(self, k_model) -> Tuple[Dict[str, NodeResponse], Model]:
         inputs_to_transpose = []
         if self._should_transform_inputs:
             inputs_to_transpose = [k_input.name for k_input in k_model.inputs]
@@ -85,7 +89,25 @@ class ModelParser:
         model_schema = json.loads(converted_k_model.to_json())
         model_schema = replace_dots_in_model_schema(model_schema)
 
-        return model_schema
+        return model_schema, converted_k_model
+
+
+def replace_dots_for_tf_ops_inbound_nodes(layer, dot_input_names):
+    inbound_nodes_to_enumerate = KerasJsonModelImport.prepare_inbound_nodes(layer['inbound_nodes'][0])
+    if len(inbound_nodes_to_enumerate) == 1:
+        if layer['inbound_nodes'][0][0] in dot_input_names:
+            layer['inbound_nodes'][0][0] = layer['inbound_nodes'][0][0].replace(".", "_")
+    elif len(inbound_nodes_to_enumerate) == 2:
+        if layer['inbound_nodes'][0][0] in dot_input_names:
+            layer['inbound_nodes'][0][0] = layer['inbound_nodes'][0][0].replace(".", "_")
+        try:
+            if layer['inbound_nodes'][0][-1]['y'][0] in dot_input_names:
+                layer['inbound_nodes'][0][0] = layer['inbound_nodes'][0][0].replace(".", "_")
+        except KeyError:
+            if layer['inbound_nodes'][0][-1]['b'][0] in dot_input_names:
+                layer['inbound_nodes'][0][0] = layer['inbound_nodes'][0][0].replace(".", "_")
+    else:
+        raise Exception('unsupported inbound_nodes')
 
 
 def replace_dots_in_model_schema(model_schema: dict) -> dict:
@@ -98,6 +120,10 @@ def replace_dots_in_model_schema(model_schema: dict) -> dict:
         if layer["name"] in dot_input_names:
             layer["name"] = layer["name"].replace(".", "_")
             layer["config"]["name"] = layer["config"]["name"].replace(".", "_")
+
+        if layer["inbound_nodes"] and not isinstance(layer["inbound_nodes"][0][0], list):
+            replace_dots_for_tf_ops_inbound_nodes(layer, dot_input_names)
+            continue
         for inbound_nodes in layer["inbound_nodes"]:
             for inbound_node in inbound_nodes:
                 if inbound_node[0] in dot_input_names:
